@@ -38,7 +38,7 @@ class REM:
         fcx = 1000.0, fcy = 1000.0, fcz = 1000.0,
         bfactor_posre = 0.5, add_bonds=True, sort=False, version='v4',
         cospower=2, turn_off_torsion_warning=False,
-        nsteps=10000,
+        nsteps=10000, rem_nsteps=0,
         turn_off_EMNVT=False,
         T=310):
         
@@ -66,7 +66,8 @@ class REM:
         self.rcut  = rcut
         self.cospower = cospower
         self.nsteps   = nsteps
-        u          = Universe(structure)
+        self.rem_nsteps = rem_nsteps
+        u = Universe(structure)
         self.protein = protein
 
 
@@ -228,17 +229,22 @@ class REM:
             print('using REM version 3')
             self.updateCustomBondForce()
             self.removeForces(['LennardJones', 'NonbondedForce']) #LennardJones14
-        else:
+        elif version == 'v4':
             print('using REM version 4')
             self.system.addForce(self.updateCustomNonbondedForce(excl=3))
             self.updateCustomBondForce()
             self.removeForces(['LennardJones', 'NonbondedForce']) #LennardJones14
- 
+        elif version == 'v5':
+            print('using REM version 5')
+            self.updateCustomNonbondedForce5()
+            self.updateCustomBondForce()
+
         print("Adding Isomer Torsions - started")
         self.system.addForce(addPeptideTorsions(  u, Kpeptide))
         self.system.addForce(addCisTransTorsions( u, Kcistrans, mapping, turn_off_torsion_warning=turn_off_torsion_warning))
         self.system.addForce(addChiralTorsions(   u, Kchiral,   mapping, turn_off_torsion_warning=turn_off_torsion_warning))
         self.system.addForce(addDihedralTorsions( u, Kdihedral, mapping, turn_off_torsion_warning=turn_off_torsion_warning))
+        self.system.addForce(addAntiDihedralTorsions( u, Kdihedral, mapping, turn_off_torsion_warning=turn_off_torsion_warning))
         print("Adding Isomer Torsions - finished")
 
 
@@ -249,7 +255,8 @@ class REM:
 
         ### Run REM without Additional Torsions
         print("Running REM without isomeric torsions")
-        self.removeForces(['PeptideTorsion', 'CisTransTorsion', 'ChiralTorsion'])
+        self.removeForces(['PeptideTorsion', 'CisTransTorsion', 'ChiralTorsion', 'DihedralTorsion', 'AntiDihedralTorsion'])
+        #self.removeForces(['PeptideTorsion', 'CisTransTorsion', 'ChiralTorsion'])
         self.runREM()
         u.atoms[['x','y','z']] = self.numpypositions
 
@@ -266,7 +273,7 @@ class REM:
             self.runEMNVT()
         elif rock:
             print("EM+NVT is turned off with ROCK (AA)")
-
+        
         ### Save
         u.atoms[['x','y','z']] = self.numpypositions
 
@@ -320,6 +327,28 @@ class REM:
                     self.system.removeForce(i)
 
 
+    def updateCustomNonbondedForce5(self):
+        forces = { force.__class__.__name__ : force for force in self.system.getForces() }
+        cnf = forces['CustomNonbondedForce']
+        # acoef/r^12 - bcoef/r^6 = 4es^12/r^12 - 4es^6/r^6
+
+        cnf.setEnergyFunction(f"min(rep, LJ); \
+            rep  = A * (cos(pi/2 * r/sig))^2; \
+            LJ   = 4 * eps * ((sig/r)^12-(sig/r)^6); \
+            eps  = bcoef(type1, type2)^2 / acoef(type1, type2) / 4 ; \
+            sig  = (acoef(type1, type2) / bcoef(type1, type2))^(1/6);")
+
+        cnf.addGlobalParameter('pi', 3.141592)
+        cnf.addGlobalParameter('A',    self.A * kilojoule/mole)
+        cnf.addGlobalParameter('rcut', self.rcut * nanometer)
+        #cnf.addGlobalParameter('rcut', cnf.getCutoffDistance())
+
+        # Original Nonbonded Force (only carries charges for charmm36)
+        onf = forces['NonbondedForce']
+        for i in range(onf.getNumParticles()):
+            onf.setParticleParameters(i, 0.0, 1.0, 0.0)
+
+
     def updateCustomNonbondedForce(self, excl=3):
         forces = { force.__class__.__name__ : force for force in self.system.getForces() }
 
@@ -449,7 +478,6 @@ class REM:
         return customforce
 
 
-
     def runREM(self):
         integrator = LangevinIntegrator(self.T*kelvin, 1/picosecond, 0.002*picoseconds)
         simulation = Simulation(self.final.topology, self.system, integrator)
@@ -460,6 +488,9 @@ class REM:
         print("E0: %.3e kJ/mol" %simulation.context.getState(getEnergy=True).getPotentialEnergy()._value)
         simulation.minimizeEnergy()
         print("E1: %.3e kJ/mol" %simulation.context.getState(getEnergy=True).getPotentialEnergy()._value)
+        if self.rem_nsteps > 0:
+            simulation.step(self.rem_nsteps)
+            print("E2: %.3e kJ/mol" %simulation.context.getState(getEnergy=True).getPotentialEnergy()._value)
         print("-------------------------------")
 
         self.positions      = simulation.context.getState(getPositions=True).getPositions()
