@@ -82,7 +82,7 @@ def runEM(structure, forcefield, out=None, nonbondedCutoff=1.1, nonbondedMethod=
         u.atoms[['x','y','z']] = pos
         u.write(out)
 
-def runEMNPT(structure, forcefield, out=None, nonbondedCutoff=1.1, nonbondedMethod='CutoffNonPeriodic',
+def runEMNPT(structure, forcefield, out=None, nonbondedCutoff=1.1, nonbondedMethod='CutoffPeriodic',
              nsteps=10000, dcdfreq=1000, csvfreq=1000, dt=0.002, P=1.0, T=310, semiisotropic=False,
              addForces=[], barfreq=100, frictionCoeff=1.0):
     '''
@@ -99,7 +99,13 @@ def runEMNPT(structure, forcefield, out=None, nonbondedCutoff=1.1, nonbondedMeth
         forcefield = ForceField(*forcefield)
     else:
         forcefield = ForceField(forcefield)
-    system = forcefield.createSystem(pdb.topology)
+
+    if nonbondedMethod == 'CutoffPeriodic':
+        nonbondedMethod = CutoffPeriodic
+    else:
+        nonbondedMethod = CutoffNonPeriodic
+
+    system = forcefield.createSystem(pdb.topology, nonbondedMethod=nonbondedMethod)
 
     ### ADD BAROSTAT
     if barfreq > 0:
@@ -466,7 +472,7 @@ def addPeptideTorsions(u, Kpeptide):
         ctf.addTorsion(CAatom0Index, CatomIndex, NatomIndex, CAatom1Index, [Kpeptide, 3.141592])
         N += 1
 
-    print('Adding PeptideTorsion for {:d} isomers'.format(N))
+    print(f'Adding PeptideTorsion for {N:d} isomers with K={Kpeptide:.2f}')
     return ctf
 
 
@@ -520,7 +526,7 @@ def addCisTransTorsions(u, Kcistrans, mapping, exclude=[], turn_off_torsion_warn
                     elif isomer == 'trans':
                         ctf.addTorsion(a, b, c, d, [Kcistrans, 3.141592])
 
-    print('Adding CisTransTorsion for {:d} isomers'.format(N))
+    print(f'Adding CisTransTorsion for {N:d} isomers with K={Kcistrans:.2f}')
     return ctf
 
 
@@ -573,7 +579,60 @@ def addDihedralTorsions(u, Kdihedral, mapping, exclude=[], turn_off_torsion_warn
                     N += 1
                     ctf.addTorsion(a, b, c, d, [Kdihedral, float(dihe[4]) * 3.141592 / 180])
 
-    print('Adding DihedralTorsion for {:d} isomers'.format(N))
+    print(f'Adding DihedralTorsion for {N:d} isomers with K={Kdihedral:.2f}')
+    return ctf
+
+
+def addAntiDihedralTorsions(u, Kdihedral, mapping, exclude=[], turn_off_torsion_warning=False):
+    if not isinstance(exclude, list):
+        exclude = [exclude]
+
+    ctf = CustomTorsionForce("k * (acos(0.999 * cos(theta-theta0)))^2")
+    ctf.setName('AntiDihedralTorsion')
+    ctf.addPerTorsionParameter("k")
+    ctf.addPerTorsionParameter("theta0")
+
+
+    N = 0
+    for resname in set(u.atoms.resname):
+        if resname in exclude: continue
+        if resname not in mapping.RESI.keys():
+            if resname == 'TIP3':
+                continue
+            elif resname.startswith('ROCK'):
+                continue
+            else:
+                print(f'Warning: {resname} not in the mapping scheme - skipping AntiDihedralTorsions for this residue')
+                continue
+
+        dihedrals = mapping.RESI[resname]['antidihedral']
+        for dihedral in dihedrals:
+            for ii in range(len(dihedral) // 5):
+                dihe = dihedral[ii * 5 : (ii+1) * 5]
+
+                bA  = u.atoms.resname == resname
+                bA0 = u.atoms.name    == dihe[0]
+                bA1 = u.atoms.name    == dihe[1]
+                bA2 = u.atoms.name    == dihe[2]
+                bA3 = u.atoms.name    == dihe[3]
+
+                atomA = u.atoms[bA & bA0].index
+                atomB = u.atoms[bA & bA1].index
+                atomC = u.atoms[bA & bA2].index
+                atomD = u.atoms[bA & bA3].index
+
+                checkbA = not (len(atomA) == len(atomB) == len(atomC) == len(atomD))
+                if checkbA and turn_off_torsion_warning: continue
+
+                assert len(atomA) == len(atomB) == \
+                    len(atomC) == len(atomD), \
+                    "the length of atoms for dihedral torsions is different"
+
+                for a, b, c, d in zip(atomA, atomB, atomC, atomD):
+                    N += 1
+                    ctf.addTorsion(a, b, c, d, [-Kdihedral, float(dihe[4]) * 3.141592 / 180])
+
+    print(f'Adding AntiDihedralTorsion for {N:d} isomers with K={Kdihedral:.2f}')
     return ctf
 
 
